@@ -64,7 +64,7 @@ static inline wchar_t *create_wchar_path(const char *utf8)
     count;
 
   wchar_t
-    *wideChar;
+    *wide;
 
   count=MultiByteToWideChar(CP_UTF8,0,utf8,-1,NULL,0);
   if ((count > MAX_PATH) && (strncmp(utf8,"\\\\?\\",4) != 0) &&
@@ -79,7 +79,7 @@ static inline wchar_t *create_wchar_path(const char *utf8)
 
       (void) FormatLocaleString(buffer,MagickPathExtent,"\\\\?\\%s",utf8);
       count+=4;
-      longPath=(wchar_t *) AcquireQuantumMemory(count,sizeof(*longPath));
+      longPath=(wchar_t *) NTAcquireQuantumMemory(count,sizeof(*longPath));
       if (longPath == (wchar_t *) NULL)
         return((wchar_t *) NULL);
       count=MultiByteToWideChar(CP_UTF8,0,buffer,-1,longPath,count);
@@ -88,21 +88,15 @@ static inline wchar_t *create_wchar_path(const char *utf8)
       longPath=(wchar_t *) RelinquishMagickMemory(longPath);
       if ((count < 5) || (count >= MAX_PATH))
         return((wchar_t *) NULL);
-      wideChar=(wchar_t *) AcquireQuantumMemory((size_t) count-3,
-        sizeof(*wideChar));
-      wcscpy(wideChar,shortPath+4);
-      return(wideChar);
+      wide=(wchar_t *) NTAcquireQuantumMemory((size_t) count-3,sizeof(*wide));
+      wcscpy(wide,shortPath+4);
+      return(wide);
     }
-  wideChar=(wchar_t *) AcquireQuantumMemory(count,sizeof(*wideChar));
-  if (wideChar == (wchar_t *) NULL)
-    return((wchar_t *) NULL);
-  count=MultiByteToWideChar(CP_UTF8,0,utf8,-1,wideChar,count);
-  if (count == 0)
-    {
-      wideChar=(wchar_t *) RelinquishMagickMemory(wideChar);
-      return((wchar_t *) NULL);
-    }
-  return(wideChar);
+  wide=(wchar_t *) NTAcquireQuantumMemory(count,sizeof(*wide));
+  if ((wide != (wchar_t *) NULL) &&
+      (MultiByteToWideChar(CP_UTF8,0,utf8,-1,wide,count) == 0))
+    wide=(wchar_t *) RelinquishMagickMemory(wide);
+  return(wide);
 }
 
 static inline wchar_t *create_wchar_mode(const char *mode)
@@ -111,28 +105,29 @@ static inline wchar_t *create_wchar_mode(const char *mode)
     count;
 
   wchar_t
-    *wideChar;
+    *wide;
 
   count=MultiByteToWideChar(CP_UTF8,0,mode,-1,NULL,0);
-  wideChar=(wchar_t *) AcquireQuantumMemory((size_t) count+1,
-    sizeof(*wideChar));
-  if (wideChar == (wchar_t *) NULL)
+  wide=(wchar_t *) AcquireQuantumMemory((size_t) count+1,
+    sizeof(*wide));
+  if (wide == (wchar_t *) NULL)
     return((wchar_t *) NULL);
-  count=MultiByteToWideChar(CP_UTF8,0,mode,-1,wideChar,count);
-  if (count == 0)
+  if (MultiByteToWideChar(CP_UTF8,0,mode,-1,wide,count) == 0)
     {
-      wideChar=(wchar_t *) RelinquishMagickMemory(wideChar);
+      wide=(wchar_t *) RelinquishMagickMemory(wide);
       return((wchar_t *) NULL);
     }
   /* Specifies that the file is not inherited by child processes */
-  wideChar[count] = L'\0';
-  wideChar[count-1] = L'N';
-  return(wideChar);
+  wide[count] = L'\0';
+  wide[count-1] = L'N';
+  return(wide);
 }
 #endif
 
 static inline int access_utf8(const char *path,int mode)
 {
+  if (path == (const char *) NULL)
+    return(-1);
 #if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__)
   return(access(path,mode));
 #else
