@@ -58,6 +58,7 @@
 #include "MagickCore/list.h"
 #include "MagickCore/magick.h"
 #include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
 #include "MagickCore/monitor.h"
 #include "MagickCore/monitor-private.h"
 #include "MagickCore/option.h"
@@ -66,7 +67,7 @@
 #include "MagickCore/static.h"
 #include "MagickCore/statistic.h"
 #include "MagickCore/string_.h"
-#include "MagickCore/module.h"
+#include "MagickCore/token.h"
 #include "coders/txt.h"
 
 /*
@@ -192,11 +193,9 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -384,53 +383,27 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
 static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
-    colorspace[MagickPathExtent],
-    text[MagickPathExtent];
-
-  double
-    max_value,
-    x_offset,
-    y_offset;
+    text[MagickPathExtent] = "\0";
 
   Image
     *image;
 
-  PixelInfo
-    pixel;
-
   MagickBooleanType
     status;
 
-  QuantumAny
-    range;
-
   ssize_t
-    i,
-    x;
-
-  Quantum
-    *q;
-
-  ssize_t
-    count,
-    type,
     y;
 
-  unsigned long
-    depth,
-    height,
-    width;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -438,26 +411,46 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  (void) memset(text,0,sizeof(text));
   (void) ReadBlobString(image,text);
   if (LocaleNCompare((char *) text,MagickTXTID,strlen(MagickTXTID)) != 0)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  x_offset=(-1.0);
-  y_offset=(-1.0);
-  q=(Quantum *) NULL;
   do
   {
-    width=0;
-    height=0;
-    max_value=0.0;
-    *colorspace='\0';
-    count=(ssize_t) sscanf(text+32,"%lu,%lu,%lf,%32s",&width,&height,&max_value,
-      colorspace);
-    if ((count != 4) || (width == 0) || (height == 0) || (max_value == 0.0))
+    char
+      colorspace[MagickPathExtent] = "\0";
+
+    double
+      max_value = 0.0;
+
+    QuantumAny
+      range = 0;
+
+    ssize_t
+      count = 0,
+      i,
+      type = 0;
+
+    unsigned long
+      depth = 0,
+      height = 0,
+      number_meta_channels = 0,
+      width = 0;
+
+    count=(ssize_t) sscanf(text+32,"%lu,%lu,%lu,%lf,%32s",&width,&height,
+      &number_meta_channels,&max_value,colorspace);
+    if (count < 5)
+      {
+        number_meta_channels=0;
+        count=(ssize_t) sscanf(text+32,"%lu,%lu,%lf,%32s",&width,&height,
+          &max_value,colorspace);
+      }
+    if ((count < 4) || (width == 0) || (height == 0) || (max_value == 0.0) ||
+        (number_meta_channels >= (unsigned long) (MaxPixelChannels-MetaPixelChannel)))
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     image->columns=width;
     image->rows=height;
-    if ((max_value == 0.0) || (max_value > 18446744073709551615.0))
+    image->number_meta_channels=number_meta_channels;
+    if ((max_value == 0.0) || (max_value >= 18446744073709551615.0))
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     for (depth=1; ((double) GetQuantumRange(depth)+1) < max_value; depth++) ;
     image->depth=depth;
@@ -479,110 +472,63 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     (void) SetImageColorspace(image,(ColorspaceType) type,exception);
     (void) SetImageBackgroundColor(image,exception);
-    GetPixelInfo(image,&pixel);
     range=GetQuantumRange(image->depth);
     status=MagickTrue;
     for (y=0; y < (ssize_t) image->rows; y++)
     {
-      double
-        alpha,
-        black,
-        blue,
-        green,
-        red;
+      ssize_t
+        x;
 
       if (status == MagickFalse)
         break;
-      red=0.0;
-      green=0.0;
-      blue=0.0;
-      black=0.0;
-      alpha=0.0;
       for (x=0; x < (ssize_t) image->columns; x++)
       {
+        char
+          token[MagickPathExtent] = "\0";
+
+        const char
+          *p;
+
+        double
+          channels[MaxPixelChannels+2] = { 0 };
+
+        Quantum
+          *q;
+
+        ssize_t
+          n;
+
+        /*
+          Read one pixel per line in this format:
+            (x-offset,y-offset) (red,green,blue,...) #HEX #CSS
+        */
         if (ReadBlobString(image,text) == (char *) NULL)
           {
             status=MagickFalse;
             break;
           }
-        switch (image->colorspace)
+        n=0;
+        for (p=text; (*p != ')') && (*p != '\0') && (n < MaxPixelChannels); )
         {
-          case LinearGRAYColorspace:
-          case GRAYColorspace:
-          {
-            if (image->alpha_trait != UndefinedPixelTrait)
-              {
-                count=(ssize_t) sscanf(text,"%lf,%lf: (%lf%*[%,]%lf%*[%,]",
-                  &x_offset,&y_offset,&red,&alpha);
-                green=red;
-                blue=red;
-                break;
-              }
-            count=(ssize_t) sscanf(text,"%lf,%lf: (%lf%*[%,]",&x_offset,
-              &y_offset,&red);
-            green=red;
-            blue=red;
-            break;
-          }
-          case CMYKColorspace:
-          {
-            if (image->alpha_trait != UndefinedPixelTrait)
-              {
-                count=(ssize_t) sscanf(text,
-                  "%lf,%lf: (%lf%*[%,]%lf%*[%,]%lf%*[%,]%lf%*[%,]%lf%*[%,]",
-                  &x_offset,&y_offset,&red,&green,&blue,&black,&alpha);
-                break;
-              }
-            count=(ssize_t) sscanf(text,
-              "%lf,%lf: (%lf%*[%,]%lf%*[%,]%lf%*[%,]%lf%*[%,]",&x_offset,
-              &y_offset,&red,&green,&blue,&black);
-            break;
-          }
-          default:
-          {
-            if (image->alpha_trait != UndefinedPixelTrait)
-              {
-                count=(ssize_t) sscanf(text,
-                  "%lf,%lf: (%lf%*[%,]%lf%*[%,]%lf%*[%,]%lf%*[%,]",
-                  &x_offset,&y_offset,&red,&green,&blue,&alpha);
-                break;
-              }
-            count=(ssize_t) sscanf(text,"%lf,%lf: (%lf%*[%,]%lf%*[%,]%lf%*[%,]",
-              &x_offset,&y_offset,&red,&green,&blue);
-            break;
-          }
+          (void) GetNextToken(p,&p,MagickPathExtent,token);
+          if (isdigit((int) ((unsigned char) *token)) != 0)
+            {
+              char
+                *end = (char *) NULL;
+
+              channels[n]=strtod(token,&end);
+              (void) GetNextToken(p,&p,MagickPathExtent,token);
+              if (*end == '%')
+                channels[n]*=0.01*range;
+              n++;
+            }
         }
-        if (strchr(text,'%') != (char *) NULL)
-          {
-            red*=0.01*range;
-            green*=0.01*range;
-            blue*=0.01*range;
-            black*=0.01*range;
-            alpha*=0.01*range;
-          }
-        if (image->colorspace == LabColorspace)
-          {
-            green+=(range+1)/2.0;
-            blue+=(range+1)/2.0;
-          }
-        pixel.red=(MagickRealType) ScaleAnyToQuantum((QuantumAny)
-          MagickMax(red+0.5,0.0),range);
-        pixel.green=(MagickRealType) ScaleAnyToQuantum((QuantumAny)
-          MagickMax(green+0.5,0.0),range);
-        pixel.blue=(MagickRealType) ScaleAnyToQuantum((QuantumAny)
-          MagickMax(blue+0.5,0.0),range);
-        pixel.black=(MagickRealType) ScaleAnyToQuantum((QuantumAny)
-          MagickMax(black+0.5,0.0),range);
-        pixel.alpha=(MagickRealType) ScaleAnyToQuantum((QuantumAny)
-          MagickMax(alpha+0.5,0.0),range);
-        q=GetAuthenticPixels(image,CastDoubleToLong(x_offset),
-          CastDoubleToLong(y_offset),1,1,exception);
+        q=GetAuthenticPixels(image,CastDoubleToLong(channels[0]),
+          CastDoubleToLong(channels[1]),1,1,exception);
         if (q == (Quantum *) NULL)
-          {
-            status=MagickFalse;
-            break;
-          }
-        SetPixelViaPixelInfo(image,&pixel,q);
+          break;
+        for (i=0; i < (ssize_t) GetImageChannels(image); i++)
+          q[i]=ScaleAnyToQuantum(CastDoubleToQuantumAny(channels[i+2]),range);
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           {
             status=MagickFalse;
@@ -613,7 +559,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
   } while (LocaleNCompare((char *) text,MagickTXTID,strlen(MagickTXTID)) == 0);
   (void) CloseBlob(image);
-  if (q == (Quantum *) NULL)
+  if (y < (ssize_t) image->rows)
     return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
@@ -721,31 +667,14 @@ ModuleExport void UnregisterTXTImage(void)
 static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image,
   ExceptionInfo *exception)
 {
-  char
-    buffer[MagickPathExtent],
-    colorspace[MagickPathExtent],
-    tuple[MagickPathExtent];
-
   MagickBooleanType
     status;
 
   MagickOffsetType
     scene;
 
-  PixelInfo
-    pixel;
-
-  const Quantum
-    *p;
-
-  ssize_t
-    x;
-
   size_t
-    imageListLength;
-
-  ssize_t
-    y;
+    number_images;
 
   /*
     Open output image file.
@@ -754,48 +683,64 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   scene=0;
-  imageListLength=GetImageListLength(image);
+  number_images=GetImageListLength(image);
   do
   {
-    ComplianceType
-      compliance;
+    char
+      buffer[MagickPathExtent],
+      colorspace[MagickPathExtent];
 
-    const char
-      *value;
+    PixelInfo
+      pixel;
+
+    QuantumAny
+      range = 0;
+
+    ssize_t
+      y;
 
     (void) CopyMagickString(colorspace,CommandOptionToMnemonic(
       MagickColorspaceOptions,(ssize_t) image->colorspace),MagickPathExtent);
     LocaleLower(colorspace);
     image->depth=GetImageQuantumDepth(image,MagickTrue);
+    range=GetQuantumRange(image->depth);
     if (image->alpha_trait != UndefinedPixelTrait)
       (void) ConcatenateMagickString(colorspace,"a",MagickPathExtent);
-    compliance=NoCompliance;
-    value=GetImageOption(image_info,"txt:compliance");
-    if (value != (char *) NULL)
-      compliance=(ComplianceType) ParseCommandOption(MagickComplianceOptions,
-        MagickFalse,value);
     if (LocaleCompare(image_info->magick,"SPARSE-COLOR") != 0)
       {
         (void) FormatLocaleString(buffer,MagickPathExtent,
-          "# ImageMagick pixel enumeration: %.20g,%.20g,%.20g,%s\n",(double)
-          image->columns,(double) image->rows,(double)
-          GetQuantumRange(image->depth),colorspace);
+          "# ImageMagick pixel enumeration: %.20g,%.20g,%.20g,%.20g,%s\n",
+          (double) image->columns,(double) image->rows,
+          (double) image->number_meta_channels,
+          (double) GetQuantumRange(image->depth),colorspace);
         (void) WriteBlobString(image,buffer);
       }
     GetPixelInfo(image,&pixel);
     for (y=0; y < (ssize_t) image->rows; y++)
     {
+      const Quantum
+        *p;
+
+      ssize_t
+        x;
+
       p=GetVirtualPixels(image,0,y,image->columns,1,exception);
       if (p == (const Quantum *) NULL)
         break;
       for (x=0; x < (ssize_t) image->columns; x++)
       {
+        char
+          tuple[MagickPathExtent];
+
+        ssize_t
+          i;
+
         GetPixelInfoPixel(image,p,&pixel);
         if (LocaleCompare(image_info->magick,"SPARSE-COLOR") == 0)
           {
@@ -804,55 +749,35 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image,
             */
             if (GetPixelAlpha(image,p) == (Quantum) OpaqueAlpha)
               {
-                GetColorTuple(&pixel,MagickFalse,tuple);
                 (void) FormatLocaleString(buffer,MagickPathExtent,
                   "%.20g,%.20g,",(double) x,(double) y);
+                GetColorTuple(&pixel,MagickFalse,tuple);
+                (void) ConcatenateMagickString(buffer,tuple,MagickPathExtent);
+                (void) ConcatenateMagickString(buffer," ",MagickPathExtent);
                 (void) WriteBlobString(image,buffer);
-                (void) WriteBlobString(image,tuple);
-                (void) WriteBlobString(image," ");
               }
             p+=GetPixelChannels(image);
             continue;
           }
-        (void) FormatLocaleString(buffer,MagickPathExtent,"%.20g,%.20g: ",
+        (void) FormatLocaleString(buffer,MagickPathExtent,"%.20g,%.20g: (",
           (double) x,(double) y);
-        (void) WriteBlobString(image,buffer);
-        (void) CopyMagickString(tuple,"(",MagickPathExtent);
-        if ((pixel.colorspace == LinearGRAYColorspace) ||
-            (pixel.colorspace == GRAYColorspace))
-          ConcatenateColorComponent(&pixel,GrayPixelChannel,compliance,tuple);
-        else
-          {
-            ConcatenateColorComponent(&pixel,RedPixelChannel,compliance,tuple);
-            (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
-            ConcatenateColorComponent(&pixel,GreenPixelChannel,compliance,
-              tuple);
-            (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
-            ConcatenateColorComponent(&pixel,BluePixelChannel,compliance,
-              tuple);
-          }
-        if (pixel.colorspace == CMYKColorspace)
-          {
-            (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
-            ConcatenateColorComponent(&pixel,BlackPixelChannel,compliance,
-              tuple);
-          }
-        if (pixel.alpha_trait != UndefinedPixelTrait)
-          {
-            (void) ConcatenateMagickString(tuple,",",MagickPathExtent);
-            ConcatenateColorComponent(&pixel,AlphaPixelChannel,compliance,
-              tuple);
-          }
-        (void) ConcatenateMagickString(tuple,")",MagickPathExtent);
-        (void) WriteBlobString(image,tuple);
-        (void) WriteBlobString(image,"  ");
+        for (i=0; i < (ssize_t) GetImageChannels(image); i++)
+        {
+          QuantumAny channel = ScaleQuantumToAny(p[i],range);
+          (void) FormatLocaleString(tuple,MagickPathExtent,"%.20g",
+            (double) channel);
+          (void) ConcatenateMagickString(buffer,tuple,MagickPathExtent);
+          if ((i+1) < (ssize_t) GetImageChannels(image))
+            (void) ConcatenateMagickString(buffer,",",MagickPathExtent);
+        }
+        (void) ConcatenateMagickString(buffer,")  ",MagickPathExtent);
         GetColorTuple(&pixel,MagickTrue,tuple);
-        (void) FormatLocaleString(buffer,MagickPathExtent,"%s",tuple);
-        (void) WriteBlobString(image,buffer);
-        (void) WriteBlobString(image,"  ");
+        (void) ConcatenateMagickString(buffer,tuple,MagickPathExtent);
+        (void) ConcatenateMagickString(buffer,"  ",MagickPathExtent);
         (void) QueryColorname(image,&pixel,SVGCompliance,tuple,exception);
-        (void) WriteBlobString(image,tuple);
-        (void) WriteBlobString(image,"\n");
+        (void) ConcatenateMagickString(buffer,tuple,MagickPathExtent);
+        (void) ConcatenateMagickString(buffer,"\n",MagickPathExtent);
+        (void) WriteBlobString(image,buffer);
         p+=GetPixelChannels(image);
       }
       status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
@@ -863,7 +788,7 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image,
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
+    status=SetImageProgress(image,SaveImagesTag,scene++,number_images);
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
